@@ -4,7 +4,6 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     2003-02-12
-// RCS-ID:      $Id: access.cpp 62067 2009-09-24 10:15:06Z JS $
 // Copyright:   (c) 2003 Julian Smart
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -87,7 +86,7 @@ private:
     wxVariant m_variant;  // List of further variants
     int       m_nCurrent; // Current enum position
 
-    DECLARE_NO_COPY_CLASS(wxIEnumVARIANT)
+    wxDECLARE_NO_COPY_CLASS(wxIEnumVARIANT);
 };
 
 // ----------------------------------------------------------------------------
@@ -339,7 +338,7 @@ private:
     wxAccessible *m_pAccessible;      // pointer to C++ class we belong to
     bool m_bQuiescing;                // Object is to be deleted
 
-    DECLARE_NO_COPY_CLASS(wxIAccessible)
+    wxDECLARE_NO_COPY_CLASS(wxIAccessible);
 };
 
 // ============================================================================
